#include "profile.h"
#include <opencv2/imgcodecs.hpp>
#include <QStandardPaths>
#include <QFile>
#include <QDir>
#include <QDebug>
#include <uvosunwrap/unwrap.h>

void CameraSetup::store() const
{
	if(darkmap.data)
		cv::imwrite(darkmapLocation(id).toStdString(), darkmap);
	if(remapMap.xMat.data)
		saveRemapMap(remapMap, remapMapLocation(id).toStdString());
	if(bgmask.data)
	{
		cv::FileStorage matf(bgmaskLocation(id).toStdString(), cv::FileStorage::WRITE);
		matf<<"bgmask"<<bgmask;
		matf.release();
	}
}

void CameraSetup::loadDarkMap(const QString &filename)
{
	darkmap = cv::imread(filename.toStdString());
}

void CameraSetup::loadBgMask(const QString &filename)
{
	cv::FileStorage matf(filename.toStdString(), cv::FileStorage::READ);
	matf["bgmask"]>>bgmask;
	matf.release();
}

void CameraSetup::loadRemapMaps(const QString &filename)
{
	remapMap = loadRemapMap(filename.toStdString());
}

void CameraSetup::load(size_t cameraId)
{
	id = cameraId;
	loadDarkMap(darkmapLocation(id));
	loadRemapMaps(remapMapLocation(id));
	loadBgMask(bgmaskLocation(id));
}

QString CameraSetup::camerasLocation()
{
	return QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + "/cameras/";
}

QString CameraSetup::darkmapLocation(size_t cameraId)
{
	return camerasLocation() + QString::number(cameraId) + ".darkmap.png";
}

QString CameraSetup::bgmaskLocation(size_t cameraId)
{
	return camerasLocation() + QString::number(cameraId) + "bgmask.mat";
}

QString CameraSetup::remapMapLocation(size_t cameraId)
{
	return camerasLocation() + QString::number(cameraId) + "remap.mat";
}

void LightingSetup::store(QSettings& settings) const
{
	settings.setValue(GROUP + QString("/brightness"), brightness);
	settings.setValue(GROUP + QString("/mask"), mask);
}

void LightingSetup::load(const QSettings& settings)
{
	brightness = settings.value(GROUP + QString("/brightness"), 0.25).toDouble();
	mask = settings.value(GROUP + QString("/mask"), 0).toUInt();
}

void Profile::store(QSettings& settings) const
{
	qDebug()<<"storing to "<<settings.fileName();
	lighting.store(settings);
	settings.setValue(GROUP + QString("/id"), static_cast<unsigned long long>(id));
	settings.setValue(GROUP + QString("/exposureTime"), exposureTime);
	settings.setValue(GROUP + QString("/name"), name_);

	if(lightmap.data)
		cv::imwrite((profileLocation() + QString::number(id) + ".lightmap.png").toStdString(), lightmap);

	settings.beginWriteArray(GROUP + QString("/cameras"), cameras.size());
	for(size_t i = 0; i < cameras.size(); ++i)
	{
		const CameraSetup& camera = cameras[i];
		settings.setArrayIndex(i);
		camera.store();
		settings.setValue("id", static_cast<unsigned long long>(camera.id));
	}
	settings.endArray();
}

void Profile::load(QSettings &settings)
{
	lighting.load(settings);
	id = settings.value(GROUP + QString("/id")).toULongLong();
	exposureTime = settings.value(GROUP + QString("/exposureTime"), 1.0/60.0).toDouble();
	name_ = settings.value(GROUP + QString("/name"), "NULL").toString();

	lightmap = cv::imread((profileLocation() + QString::number(id) + ".lightmap.png").toStdString());

	int size = settings.beginReadArray(GROUP + QString("/cameras"));
	for(int i = 0; i < size; ++i)
	{
		settings.setArrayIndex(i);
		cameras.push_back(CameraSetup());
		cameras.back().load(settings.value("id", 0).toULongLong());
	}
	settings.endArray();
}

void Profile::load(const QString& name)
{
	qDebug()<<profileLocation();
	QSettings settings(profileLocation() + name + ".profile.ini", QSettings::IniFormat);
	name_=name;
	load(settings);
}

void Profile::store(const QString& name)
{
	QSettings settings(profileLocation() + name + ".profile.ini", QSettings::IniFormat);
	name_=name;
	store(settings);
	settings.sync();
}

void Profile::load()
{
	load(name_);
}

void Profile::store()
{
	store(name_);
}

void Profile::deleteProfile()
{
	QFile::remove(profileLocation() + name_ + ".profile.ini");
}

QList<QString> Profile::avaiableProfiles()
{
	QDir dir(profileLocation());
	dir.setFilter(QDir::Files);
	QStringList filters;
	filters<<"*.ini";
	dir.setNameFilters(filters);
	QList<QString> ret;
	QStringList list = dir.entryList();

	for(QString string : list)
		ret.push_back(string.split(".")[0]);
	return  ret;
}

void Profile::setCamerasSetupsFromDescription(const std::vector<cam::Camera::Description>& desc)
{
	cameras.clear();
	cameras.reserve(desc.size());
	for(size_t i = 0; i < desc.size(); ++i)
	{
		CameraSetup tmp;
		tmp.load(desc[i].getHash());
		cameras.push_back(tmp);
	}
}

bool Profile::camerasSufficant(const std::vector<cam::Camera::Description>& desc) const
{
	bool invalid = false;
	for(auto& cameraProfile : cameras)
	{
		bool found = false;
		for(auto& camera : desc)
		{
			if(camera.getHash() == cameraProfile.id)
			{
				found = true;
				break;
			}
		}

		if(!found)
		{
			invalid = true;
			break;
		}
	}
	return  !invalid;
}


QString Profile::profileLocation()
{
	return QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + "/profiles/";
}

Profile::Profile(const QString& name): name_(name)
{
	id = QRandomGenerator::system()->generate64();
}
